'use strict';
let Metalsmith = require('metalsmith');
let Handlebars = require('handlebars');
let render = require('consolidate').handlebars.render;
let path = require('path');
let match = require('minimatch');
let async = require('async');
let getOptions = require('./options');
let ask = require('./ask');
let filter = require('./filter');

// register handlebars helper
Handlebars.registerHelper('if_eq', function (a, b, opts) {
    return a === b
        ? opts.fn(this)
        : opts.inverse(this);
});

Handlebars.registerHelper('unless_eq', function (a, b, opts) {
    return a === b
        ? opts.inverse(this)
        : opts.fn(this);
});

/**
 * Create a middleware for asking questions.
 *
 * @param {Object} prompts
 * @return {Function}
 */

function askQuestions (prompts) {
    return function (files, metalsmith, done) {
        ask(prompts, metalsmith.metadata(), done);
    };
}

/**
 * Create a middleware for filtering files.
 *
 * @param {Object} filters
 * @return {Function}
 */

function filterFiles (filters) {
    return function (files, metalsmith, done) {
        filter(files, filters, metalsmith.metadata(), done);
    };
}

/**
 * Template in place plugin.
 *
 * @param {Object} files
 * @param {Metalsmith} metalsmith
 * @param {Function} done
 */

function renderTemplateFiles (skipInterpolation) {
    return function (files, metalsmith, done) {
        let keys = Object.keys(files);
        let metalsmithMetadata = metalsmith.metadata();
        async.each(keys, function (file, next) {
            // skipping files with skipInterpolation option
            if (skipInterpolation && match(file, skipInterpolation, { dot: true })) {
                return next();
            }
            var str = files[file].contents.toString();
            // do not attempt to render files that do not have mustaches
            if (!(/{{([^{}]+)}}/g.test(str) || /{{([^{}]+)}}/g.test(file))) {
                return next();
            }
            render(str, metalsmithMetadata, function (err, res) {
                if (err) return next(err);
                files[file].contents = new Buffer(res);

                /*
                 * render filename
                 */

                if (/{{([^{}]+)}}/g.test(file)) {

                    render(file, metalsmithMetadata, function(err, res) {
                        files[res] = files[file];
                        delete files[file];
                        next();
                    });

                }else {
                    next();
                }
            });
        }, done);
    };
}

/**
 * Display template complete message.
 *
 * @param {String} message
 * @param {Object} data
 */

function logMessage (message, data) {
    if (!message) return;
    render(message, data, function (err, res) {
        if (err) {
            console.error('\n   Error when rendering template complete message: ' + err.message.trim());
        } else {
            console.log('\n' + res.split(/\r?\n/g).map(function (line) {
                return '   ' + line;
            }).join('\n'));
        }
    });
}

module.exports = function generate (name, src, dest, entry, done) {
    let opts = getOptions(name, src);
    let metalsmith = Metalsmith(path.join(src, 'template'));
    let data = Object.assign(metalsmith.metadata(), {
        destDirName: name,
        inPlace: dest === process.cwd(),
        noEscape: true
    });
    opts.helpers && Object.keys(opts.helpers).map(function (key) {
        Handlebars.registerHelper(key, opts.helpers[key]);
    });
    if(entry == 'init') {
        opts.completeMessage = opts.completeMessage || `  cookvue:  应用 '{{destDirName}}' 已生成。\n\n  请按照以下步骤启动项目:\n\n  cd {{destDirName}}\n  npm install\n  npm run dev\n`;
    }else if(entry == 'newpage') {
        opts.completeMessage = opts.completeMessage || `  cookvue:  页面 '{{destDirName}}' 已生成。\n`;
    }

    metalsmith
        .use(askQuestions(opts.prompts))
        .use(filterFiles(opts.filters))
        .use(renderTemplateFiles(opts.skipInterpolation))
        .clean(false)
        .source('.') // start from template root instead of `./src` which is Metalsmith's default for `source`
        .destination(dest)
        .build(function (err) {
            if(!err) {
                logMessage(opts.completeMessage, data);
            }
            done(err);
        });

    return data;
};
